package hdl

import (
	"applet/app/db"
	"applet/app/e"
	"applet/app/lib/validate"
	"applet/app/md"
	"applet/app/svc"
	"applet/app/utils"
	"applet/app/utils/cache"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/implement"
	"code.fnuoos.com/EggPlanet/egg_models.git/src/model"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/aliyun"
	"code.fnuoos.com/EggPlanet/egg_system_rules.git/enum"
	"fmt"
	"github.com/gin-gonic/gin"
	"time"
)

// SmsSend
// @Summary      发送短信
// @Tags         登录注册
// @Description  发送短信
// @Accept       json
// @Produce      json
// @Param        req 	body      md.SmsSendReq          true   "注册参数"
// @Success      200   {string}   			"成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/smsSend [post]
func SmsSend(c *gin.Context) {
	var req md.SmsSendReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}

	userDb := implement.NewUserDb(db.Db)
	if utils.InArr(req.Type, []string{"h5Register", "wechatBindPhone", "login"}) == false {
		user, err := userDb.UserGetOneByParams(map[string]interface{}{
			"key":   "phone",
			"value": req.Mobile,
		})
		if err != nil {
			e.OutErr(c, 400, err)
			return
		}
		if user == nil {
			e.OutErr(c, 400, e.NewErr(400, "账号不存在"))
			return
		}
	}
	param := map[string]string{
		"lot_number":     req.LotNumber,
		"gen_time":       req.GenTime,
		"captcha_output": req.CaptchaOutput,
		"pass_token":     req.PassToken,
	}
	//校验图形验证码
	id, key := svc.AliyunCaptchBase(c)
	err = aliyun.AliyunCheckCaptcha(id, key, param)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "图形验证码校验失败"))
		return
	}
	data := svc.AliyunSmsBase(c, req.Type)
	//发送短信
	err = aliyun.AliyunSendSms(data["aliyun_sms_id"], data["aliyun_sms_secret"], req.Mobile, data["aliyun_sms_sign_name"], data["aliyun_sms_code"], "")
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "发送失败"))
		return
	}
	//发送短信
	e.OutSuc(c, "success", nil)
	return
}

// FastLogin
// @Summary      一键登录
// @Tags         登录注册
// @Description  一键登录
// @Accept       json
// @Produce      json
// @Param        req 	body      md.FastLoginReq          true   "注册参数"
// @Success      200   {object}   md.LoginResponse			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/fastLogin [post]
func FastLogin(c *gin.Context) {
	var req md.FastLoginReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	data := svc.AliyunSmsBase(c, "")
	//获取手机
	phone, err := aliyun.AliyunFastPhone(data["aliyun_sms_id"], data["aliyun_sms_secret"], req.Token)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "获取手机号失败"))
		return
	}
	var reqs = md.RegisterReq{
		Mobile: phone,
		Type:   "app",
	}
	commReq(c, reqs)
}

// WechatLogin
// @Summary      微信登陆
// @Tags         登录注册
// @Description  微信登陆
// @Accept       json
// @Produce      json
// @Param        req 	body      md.WechatLoginReq          true   "注册参数"
// @Success      200   {object}   md.LoginResponse			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/wechatLogin [post]
func WechatLogin(c *gin.Context) {
	var req md.WechatLoginReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	if req.OpenID == "" {
		e.OutErr(c, 400, e.NewErr(400, "登录失败"))
		return
	}
	param := map[string]interface{}{
		"key":   "open_id",
		"value": req.OpenID,
	}
	if req.UnionId != "" {
		param = map[string]interface{}{
			"key":   "union_id",
			"value": req.UnionId,
		}
	}
	// 如果内存缓存中没有，则从 数据库查询
	newUserDb := implement.NewUserDb(db.Db)
	userModel, err := newUserDb.UserGetOneByParams(param)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "登录失败"))
		return
	}
	token := ""
	if userModel != nil {
		userModel.OpenId = req.OpenID
		userModel.Avatar = req.Avatar
		userModel.Nickname = req.Nickname
		now := time.Now()
		userModel.LastLoginAt = now.Format("2006-01-02 15:04:05")
		userModel.LastLoginIp = utils.GetIP(c.Request)
		userModel.UpdateAt = now.Format("2006-01-02 15:04:05")
		newUserDb.UpdateUser(userModel, "open_id,avatar,nickname,last_login_ip,last_login_at,update_at")
		svc.UserImeiAdd(c, userModel.Id)
		token, err = svc.HandleLoginToken(userModel)
		if err != nil {
			e.OutErr(c, e.ERR, err.Error())
			return
		}
	}
	e.OutSuc(c, md.LoginResponse{Token: token}, nil)
	return
}

// FindPassword
// @Summary      找回密码
// @Tags         登录注册
// @Description  找回密码
// @Accept       json
// @Produce      json
// @Param        req 	body      md.FindPasswordReq          true   "注册参数"
// @Success      200   string   			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/findPassword [post]
func FindPassword(c *gin.Context) {
	var req md.FindPasswordReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	userDb := implement.NewUserDb(db.Db)
	user, err := userDb.UserGetOneByParams(map[string]interface{}{
		"key":   "phone",
		"value": req.Mobile,
	})
	if user == nil {
		e.OutErr(c, 400, e.NewErr(400, "手机号不存在"))
		return
	}
	data := svc.AliyunSmsBase(c, "")
	//校验短信
	err = aliyun.AliyunCheckSms(data["aliyun_sms_id"], data["aliyun_sms_secret"], req.Mobile, req.Code)
	if err != nil {
		e.OutErr(c, 400, e.NewErr(400, "验证码错误,请重试"))
		return
	}
	user.Password = utils.Md5(req.Password)
	userDb.UpdateUser(user, "password")
	e.OutSuc(c, "success", nil)
	return
}

// Register
// @Summary      注册
// @Tags         登录注册
// @Description  注册
// @Accept       json
// @Produce      json
// @Param        req 	body      md.RegisterReq          true   "注册参数"
// @Success      200   {object}   md.LoginResponse			"登录成功返回"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/register [post]
func Register(c *gin.Context) {
	var req md.RegisterReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = svc.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	data := svc.AliyunSmsBase(c, "")
	//校验短信
	err = aliyun.AliyunCheckSms(data["aliyun_sms_id"], data["aliyun_sms_secret"], req.Mobile, req.Code)
	//if err != nil {
	//	e.OutErr(c, 400, e.NewErr(400, "验证码错误,请重试"))
	//	return
	//}
	commReq(c, req)
}

// Login 登陆
// @Summary      登陆
// @Tags         登录注册
// @Description  登入
// @Accept       json
// @Produce      json
// @Param        req 	body      md.LoginReq          true   "用户名密码"
// @Success      200   {object}   md.LoginResponse				"token"
// @Failure      400   {object}   md.Response              	"具体错误"
// @Router       /api/v1/login [post]
func Login(c *gin.Context) {
	var req md.LoginReq
	err := c.ShouldBindJSON(&req)
	if err != nil {
		err = validate.HandleValidateErr(err)
		err1 := err.(e.E)
		e.OutErr(c, err1.Code, err1.Error())
		return
	}
	userDb := implement.NewUserDb(db.Db)
	user, err := userDb.UserGetOneByParams(map[string]interface{}{
		"key":   "phone",
		"value": req.Phone,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err)
		return
	}
	if user == nil {
		e.OutErr(c, e.ERR_NO_DATA, "账号不存在！")
		return
	}
	fmt.Println(utils.Md5(req.PassWord))
	if utils.Md5(req.PassWord) != user.Password {
		e.OutErr(c, e.ERR_INVALID_ARGS, "密码错误")
		return
	}
	//ip := utils.GetIP(c.Request)
	token, err := svc.HandleLoginToken(user)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}
	svc.UserImeiAdd(c, user.Id)
	e.OutSuc(c, md.LoginResponse{
		Token: token,
	}, nil)
	return
}

// 公共处理注册
func commReq(c *gin.Context, req md.RegisterReq) {
	now := time.Now()
	userDb := implement.NewUserDb(db.Db)
	user, err := userDb.UserGetOneByParams(map[string]interface{}{
		"key":   "phone",
		"value": req.Mobile,
	})
	if err != nil {
		e.OutErr(c, e.ERR_DB_ORM, err)
		return
	}

	ip := utils.GetIP(c.Request)
	if user == nil {
		registerType := func() int {
			if req.Type == "H5" {
				return 2
			}
			return 1
		}()
		var parentUser *model.User
		var inviteCode string
		if req.InviteCode != "" {
			// Todo::拥有上下级关系
			parentUser, err = userDb.GetUserByInviteCode(req.InviteCode)
			if err != nil {
				e.OutErr(c, e.ERR_DB_ORM, err)
				return
			}
			if parentUser.Id == 0 {
				parentUser, err = userDb.GetUserByCustomInviteCode(req.InviteCode)
				if err != nil {
					e.OutErr(c, e.ERR_DB_ORM, err)
					return
				}
				e.OutErr(c, e.ERR_SUP_NOTFOUND, "上级用户不存在!")
				return
			}
			redisConn := cache.GetPool().Get()
			sysCfgDb := implement.NewSysCfgDb(db.Db, redisConn)
			sysCfg := sysCfgDb.SysCfgFindWithDb(enum.AppInviteType, enum.AppInviteLength)
			inviteCode = svc.ReturnCode(utils.StrToInt(sysCfg[enum.AppInviteLength]), utils.StrToInt(sysCfg[enum.AppInviteType]), 0)
			initLV := 1
			ur := new(model.UserRelate)
			ur.ParentUid = parentUser.Id
			ur.Uid = user.Id
			ur.Level = initLV
			ur.InviteTime = time.Now().Format("2006-01-02 15:04:05")

			userRelateDb := implement.NewUserRelateDb(db.Db)
			_, err = userRelateDb.UserRelateInsert(ur)
			if err != nil {
				e.OutErr(c, e.ERR_DB_ORM, err)
				return
			}

			// 插入多级关联
			go svc.RoutineMultiRelate(ur.ParentUid, ur.Uid, initLV)
		}
		var parentUid = func() int64 {
			if parentUser != nil {
				return parentUser.Id
			}
			return 0
		}()
		if req.Nickname == "" && req.Mobile != "" {
			appName := svc.GetSysCfgStr("app_name")
			req.Nickname = appName + req.Mobile[len(req.Mobile)-4:]
		}
		if req.Avatar == "" {
			req.Avatar = svc.GetSysCfgStr("default_avatar")
		}
		user = &model.User{
			Phone:            req.Mobile,
			UnionId:          req.UnionId,
			OpenId:           req.OpenID,
			Nickname:         req.Nickname,
			Avatar:           req.Avatar,
			Password:         "",
			Passcode:         "",
			Level:            0,
			InviteTotal:      0,
			State:            int(enum.UserStateForNormal),
			LastLoginIp:      ip,
			Sex:              req.Sex,
			ParentUid:        parentUid,
			SystemInviteCode: inviteCode,
			CustomInviteCode: "",
			Memo:             "",
			IsRealName:       0,
			RegisterType:     registerType,
			CreateAt:         now.Format("2006-01-02 15:04:05"),
			UpdateAt:         now.Format("2006-01-02 15:04:05"),
		}
		if req.Password != "" {
			user.Password = utils.Md5(req.Password)
		}

		insertAffected, err1 := userDb.UserInsert(user)
		if err1 != nil {
			e.OutErr(c, e.ERR_DB_ORM, err1)
			return
		}
		if insertAffected <= 0 {
			e.OutErr(c, e.ERR_DB_ORM, "新增用户数据失败")
			return
		}
		if parentUid > 0 {
			//TODO::推入mq异步处理
		}
	}

	token, err := svc.HandleLoginToken(user)
	if err != nil {
		e.OutErr(c, e.ERR, err.Error())
		return
	}

	if req.Nickname != "" {
		user.Nickname = req.Nickname
	}
	if req.Avatar != "" {
		user.Avatar = req.Avatar
	}
	if req.Password != "" {
		user.Password = utils.Md5(req.Password)
	}

	// 微信
	if req.OpenID != "" {
		user.OpenId = req.OpenID
	}
	if req.UnionId != "" {
		user.UnionId = req.UnionId
	}
	// 登录信息
	user.LastLoginAt = now.Format("2006-01-02 15:04:05")
	user.LastLoginIp = ip
	user.UpdateAt = now.Format("2006-01-02 15:04:05")

	updateAffected, err1 := userDb.UpdateUser(user)
	if err1 != nil {
		e.OutErr(c, e.ERR_DB_ORM, err1)
		return
	}
	if updateAffected <= 0 {
		e.OutErr(c, e.ERR_DB_ORM, "更新用户数据失败")
		return
	}
	svc.UserImeiAdd(c, user.Id)
	e.OutSuc(c, md.LoginResponse{
		Token: token,
	}, nil)
	return
}
