package md

type EggEnergyDetailReq struct {
	Page      int    `json:"page,required"`  // 页数
	Limit     int    `json:"limit,required"` // 每页大小
	Kind      string `json:"kind"`           // 类型：1. 个人蛋蛋能量 2.团队蛋蛋能量
	Direction string `json:"direction"`      // 类型 1.收入 2.支出 全部则传空
	Type      string `json:"type"`           // 变动类型 全部则传空
	Asc       string `json:"asc"`            // 1.升序 2.倒序
}

type TransferTypeNode struct {
	Value string `json:"value"`
	Name  string `json:"name"`
}

type IntegralList struct {
	IncomeList []TransferTypeNode `json:"income_list"` // 收入类型列表
	ExpendList []TransferTypeNode `json:"expend_list"` // 支出类型列表
}

type TransferTypeList struct {
	PersonalTransferTypeList IntegralList `json:"personal_transfer_type_list"` // 个人积分变动列表
	TeamTransferTypeList     IntegralList `json:"TeamTransferTypeList"`        // 团队积分变动列表
}

type EggEnergyDetailResp struct {
	PersonEggEnergy     string              `json:"person_egg_energy"`      // 个人蛋蛋能量
	TeamEggEnergy       string              `json:"team_egg_energy"`        // 团队蛋蛋能量
	TotalEggEnergy      string              `json:"total_egg_energy"`       // 总蛋蛋能量
	TotalEggEnergyPrice string              `json:"total_egg_energy_price"` // 蛋蛋能量总价
	List                []map[string]string `json:"list"`                   // 流水数据
	TransferTypeList    TransferTypeList    `json:"transfer_type_list"`     // 转账类型列表
	Paginate            Paginate            `json:"paginate"`               // 分页信息
}

type EggPointDetailsReq struct {
	Page      int    `json:"page,required"`  // 页数
	Limit     int    `json:"limit,required"` // 每页大小
	Kind      string `json:"kind"`           // 类型：1. 个人蛋蛋积分 2.团队蛋蛋积分
	Direction string `json:"direction"`      // 类型 1.收入 2.支出 全部则传空
	Type      string `json:"type"`           // 变动类型 全部则传空
	Asc       string `json:"asc"`            // 1.升序 2.倒序
}

type EggPointDetailsResp struct {
	PersonEggPoints      string              `json:"person_egg_points"`       // 个人蛋蛋积分
	TeamEggPoints        string              `json:"team_egg_points"`         // 团队蛋蛋积分
	TotalEggPoints       string              `json:"total_egg_points"`        // 总蛋蛋积分
	TotalEggPointsAmount string              `json:"total_egg_points_amount"` // 蛋蛋积分余额
	List                 []map[string]string `json:"list"`                    // 流水数据
	TransferTypeList     TransferTypeList    `json:"transfer_type_list"`      // 转账类型列表
	Paginate             Paginate            `json:"paginate"`                // 分页信息
}

type MyFansResp struct {
	NowTeamUser             int64  `json:"now_team_user"`              // 团队人数
	NowTeamAssistanceNum    int    `json:"now_team_assistance_num"`    // 团队助力人数
	NowTeamAssistanceReward string `json:"now_team_assistance_reward"` // 团队助力收益
}

type OneLineUserNode struct {
	IsOnline   bool   `json:"is_online"`
	AvatarURL  string `json:"avatar_url"`
	Nickname   string `json:"nickname"`
	InviteTime string `json:"invite_time"`
}

type MyFansUserListResp struct {
	List []OneLineUserNode `json:"list"`
}

type SpaceListNode struct {
	Name     string `json:"name"`      // 维度
	MaxCount int    `json:"max_count"` // 最大容纳人数
	NowCount int    `json:"now_count"` // 当前人数
}

type NineDimensionalSpaceResp struct {
	Rows           string          `json:"rows"`             // 行数
	Times          string          `json:"times"`            // 排数
	SpaceTotalNums int             `json:"space_total_nums"` // 空间总数
	SpaceNums      int             `json:"space_nums"`       // 空间人数
	SpaceList      []SpaceListNode `json:"space_list"`       // 数据列表
	ALotOfNums     int             `json:"a_lot_of_nums"`    // 全网至多用户数
	DayNum         int             `json:"day_num"`          // 连续x天不活跃
}

type BasalRateResp struct {
	ConsumedTime       string `json:"consumed_time"`        // 消耗时间/小时
	ConsumedEggEnergy  string `json:"consumed_egg_energy"`  // 收益蛋蛋能量
	RemainingTime      string `json:"remaining_time"`       // 剩余时间/小时
	RemainingEggEnergy string `json:"remaining_egg_energy"` // 剩余蛋蛋能量
	BasalRate          string `json:"basal_rate"`           // 基础速率
	ConsumedEggPoint   string `json:"consumed_egg_point"`   // 消耗蛋蛋积分
	EstimatedRevenue   string `json:"estimated_revenue"`    // 预估收益蛋蛋能量
	SignCountdown      string `json:"sign_countdown"`       // 收益倒计时
}

type TotalRateResp struct {
	NowBasalRate             string `json:"now_basal_rate"`              // 当前基础速率/小时
	NowTeamRate              string `json:"now_team_rate"`               // 当前团队速率/小时
	ConsumedPersonalEggPoint string `json:"consumed_personal_egg_point"` // 消耗个人蛋蛋积分
	ConsumedTeamEggPoint     string `json:"consumed_team_egg_point"`     // 消耗团队蛋蛋积分
	NowTotalRate             string `json:"now_total_rate"`              // 总速率/小时
}
